//==============================================================================
//File name:    "3dx.h"
//Purpose:      Header File	for: Win 3dx.dll, Linux 3dx.so   
//Version:      4.01
//Copyright:    (c) 2021, S3D llc, s3d.devices@gmail.com		
//==============================================================================
// The following ifdef block is the standard way of creating macros which 
// make exporting from a DLL simpler. All files within this DLL are compiled 
// with the S3DX_EXPORTS symbol defined on the command line. this symbol 
// should not be defined on any project that uses this DLL. This way any other 
// project whose source files include this file see S3D_API functions as 
// being imported from a DLL, whereas this DLL sees symbols defined with this 
// macro as being exported.
 
#ifndef _3DX_DLL_H_
#define _3DX_DLL_H_

//---- OS-systems ----
#ifdef  _WIN64
//Use  64-bit version
#define __win64
#define __windows__
#else
#ifdef  _WIN32
//Use  32-bit version
#define __win32
#define __windows__
#endif
#endif
//--------------------

//For Windows   
#ifdef __windows__				
#include <Windows.h>
#endif
//For Linux stuff
#ifdef  __linux__	 
#include <unistd.h>
#endif

#ifdef  S3D_EXPORTS
#define S3D_API __declspec(dllexport)
#else
#define S3D_API __declspec(dllimport)
#endif

#ifdef  __cplusplus
extern "C" 
{
#endif //end __cplusplus

//------------------------------------------------------------------------------
//Adapter USB-COM Hotplug function:
//------------------------------------------------------------------------------
//Use: Add func afx_msg BOOL OnDeviceChange(UINT EventType, DWORD dwData);
//Add  ON_WM_DEVICECHANGE() to application message_map list
//Put  USB_HotPlugHandler() to OnDeviceChange()

//------------------------------------------------------------------------------
//USB-COM Hotplug callback
//------------------------------------------------------------------------------
/*BOOL CCCPCDlg::OnDeviceChange(UINT EventType, DWORD dwData)
{
  USBCOM_HotPlugHandler(hUSB, EventType, dwData);  
  return TRUE;
} */

//Support Attach Detach USB dongle
S3D_API int  USB_HotPlugHandler(HANDLE hUSB, unsigned int EventType, DWORD_PTR dwData);
//pAdapterComboBox pointer come from application GUI interface, if not used = NULL
S3D_API void USB_SetHotPlug(HANDLE hUSB, HWND hWndParent, HANDLE pAdapterComboBox);

//------------------------------------------------------------------------------
//Adapter control functions:
//------------------------------------------------------------------------------
//Change Baudrate for COM-adapter, possible values: 115200, 0=AUTO
S3D_API int USBCOM_SetBaudrate(HANDLE hUSB, unsigned short baudrate);
S3D_API int USBCOM_GetBaudrate(HANDLE hUSB);

//------------------------------------------------------------------------------
//API functions:
//------------------------------------------------------------------------------
//Create 3DX-interface
S3D_API HANDLE S3D_Ini(void);

//Close 3DX-interface
S3D_API void S3D_Close(HANDLE lpHandle);

//Return connected to host PC USB-BT dongle list, and write to ComboBox control 
S3D_API int S3D_GetBtAdapterList(HANDLE lpComboBox, HANDLE pCStringArrayName);

//Call it if need cnahne USB-BT dongle adapter
S3D_API int S3D_ChangeBtAdapter(HANDLE lpComboBox, HANDLE &lpHandle);

//Open COM adapter by number, COM Port number required 
S3D_API int S3D_OpenSerial(HANDLE lpHandle, int com_number);

//Close COM adapter
S3D_API void S3D_CloseSerial(HANDLE lpHandle, int com_number);

/* for Bluetooth
//Set Msg Acceptance Mask by device ID range 
//For reset all ID, write 0. For accept all msg, write:(0,0xFFFF)
S3D_API int  S3D_SetRxFilter(HANDLE lpHandle, 
                             short S3D_ID_From, short S3D_ID_To);

//Modification Msg Acceptance Mask by add new single device ID 
S3D_API int S3D_AddRxID(HANDLE lpHandle, short DeviceID);

//Set Msg Acceptance Mask directly by bit
//For reset mask, write 0. For pass all msg, write 0xFFFF
S3D_API int S3D_SetRxMask(HANDLE lpHandle, short ID_Mask);
*/                    

//Read internal settings
S3D_API int S3D_GetDeviceSetup(HANDLE lpHandle, int device_id,
                               char *pBuffer, int length);

//Check device presence
S3D_API int S3D_CheckDevice(HANDLE lpHandle, int device_id);

//Return devices count, connected to the host computer  
S3D_API int S3D_ReturnDeviceCount(HANDLE lpHandle);

//Return devices unique ID from S3D_ReturnDeviceCount
S3D_API int S3D_GetDeviceID(HANDLE lpHandle, int count_number);

//Return devices list
S3D_API int S3D_GetDeviceList(HANDLE lpHandle, 
                              HANDLE lpListBox,	 //CListBox class pointer
                              int device_id_low, //Low range
                              int device_id_high,//Top range
                              LPVOID pIDs, 	     //CByteArray class pointer
                              LPVOID pNames);    //CStringArray class pointer

//Return current *.dll/*.so version
S3D_API int S3D_GetLibVersion(void);

//------------------------------------------------------------------------------
//Methods
//------------------------------------------------------------------------------
S3D_API int S3D_SyncGDI(HANDLE lpHandle, LPVOID pS3Dstruct);
S3D_API int S3D_SyncDX9(HANDLE lpHandle, LPVOID pS3Dstruct);
S3D_API int S3D_SyncDX11(HANDLE lpHandle, LPVOID pS3Dstruct);
S3D_API int S3D_SyncGL2(HANDLE lpHandle, LPVOID pS3Dstruct);

//Control
S3D_API int S3D_SetFPS(HANDLE lpHandle, long period_ns);
S3D_API int S3D_ShowRun(HANDLE lpHandle);
S3D_API int S3D_ShowStop(HANDLE lpHandle);
S3D_API int S3D_ResetToDefault(HANDLE lpHandle);

//Tuning
S3D_API int S3D_SetLensOffset(HANDLE lpHandle, int lens, int delay_persents);
S3D_API int S3D_SetFrameOffset(HANDLE lpHandle, int delay_persents);
S3D_API int S3D_SetShowONCount(HANDLE lpHandle, int frame_count);
S3D_API int S3D_SetShowOffCount(HANDLE lpHandle, int frame_count);

//Stability
S3D_API int S3D_SetSyncFilter(HANDLE lpHandle, int freq_band);
S3D_API int S3D_SetSyncLostLimit(HANDLE lpHandle, int factor);
S3D_API int S3D_SetJitterLimit(HANDLE lpHandle, int factor);
S3D_API int S3D_SetFrameDroppingLimit(HANDLE lpHandle, int factor);
S3D_API int S3D_SetCorrelator(HANDLE lpHandle, int param1, int param2);
S3D_API int S3D_SetCoeff(HANDLE lpHandle, int a, int b, int c, int d);

//------------------------------------------------------------------------------
//Dialog windows, Usage: S3D_xxx_Dialog(GetSafeHwnd(), p3DG);
//------------------------------------------------------------------------------
//Device main dialog +
S3D_API void S3D_MainDialog(HWND hWndParent, HANDLE lpHandle);

//Remote device firmware update dialog
S3D_API void S3D_ISP_Dialog(HWND hWndParent, HANDLE lpHandle);

//Device setup dialog +
S3D_API void S3D_GUI_Test(HWND hWndParent, HANDLE lpHandle);

//Messages from glasses
S3D_API void S3D_MessageBox(HANDLE lpHandle, HWND hWnd, const char *pTxt, int flag);

//GUI window event handler
//S3D_API void S3D_GUI_EvHandler(MSG *pMsg);

#ifdef __cplusplus
}
#endif //end __cplusplus

#endif
